from django.template import Library, Node, TemplateSyntaxError, Variable
from django.utils.translation import ugettext as _
from django.db.models import Sum

from visits.models import Visit

register = Library()


class VisitsNode(Node):
    def __init__(self, obj, context_var):
        self.obj = Variable(obj)
        self.context_var = context_var

    def render(self, context):
        obj = self.obj.resolve(context)
        context[self.context_var] = Visit.objects.filter(
                object_model=obj.__class__.__name__,
                object_id=obj.id
        ).aggregate(visits_sum=Sum('visits'))['visits_sum']
        return ''


def do_object_visits(parser, token):
    """
    Retrive the number of visits of a model/slug
    """
    bits = token.contents.split()
    if len(bits) != 4:
        raise TemplateSyntaxError(
                _('%s tag requires exactly three arguments') % bits[0]
        )
    if bits[2] != 'as':
        raise TemplateSyntaxError(
                _("second argument to %s tag must be 'as'") % bits[0]
        )
    return VisitsNode(bits[1], bits[3])

register.tag('object_visits', do_object_visits)



# -*- coding: utf-8 -*-
from django.db import models
from django.utils.translation import ugettext_lazy as _
from visits import settings
from visits.utils import is_ignored, gen_hash

from datetime import datetime, timedelta

class VisitManager(models.Manager):
    def add_uri_visit(self, request, uri):
        visitor_hash = gen_hash(request, uri)
        visit = self.get_or_create(
                visitor_hash=visitor_hash,
                uri=uri,
                ip_address=request.META.get('REMOTE_ADDR','')
        )

        if not is_ignored(request, visit[0]):
            visit[0].last_visit = datetime.today()
            visit[0].visits += 1
            visit[0].save()

    def add_object_visit(self, request, obj):
        visitor_hash = gen_hash(request, obj._meta.app_label, obj.__class__.__name__, obj.id)
        visit = self.get_or_create(
                visitor_hash=visitor_hash,
                object_app=obj._meta.app_label,
                object_model=obj.__class__.__name__,
                object_id=obj.id,
                ip_address=request.META.get('REMOTE_ADDR','')
        )

        if not is_ignored(request, visit[0]):
            visit[0].last_visit = datetime.today()
            visit[0].visits += 1
            visit[0].save()

class Visit(models.Model):
    visitor_hash = models.CharField(max_length=40, blank=True, null=True, db_index=True)
    uri = models.CharField(max_length=255)
    ip_address = models.IPAddressField(blank=True, null=True, db_index=True)
    last_visit = models.DateTimeField(blank=True, null=True)
    visits = models.IntegerField(default=0)
    object_app = models.CharField(max_length=255)
    object_model = models.CharField(max_length=255)
    object_id = models.CharField(max_length=255)
    blocked = models.BooleanField(default=False)

    objects = VisitManager()

    def __unicode__(self):
        return self.visitor_hash

    class Meta:
        app_label = "visits"
        ordering = ('uri', 'object_model', 'object_id')
        verbose_name = _('visit')
        verbose_name_plural = _('visits')



from django.contrib import admin
from visits.models import Visit


class VisitAdmin(admin.ModelAdmin):
    list_display = ['visitor_hash', 'uri', 'object_app', 'object_model', 'object_id', 'visits']
    list_filter = ['object_app']
    search_fields = ['object_app', 'object_model', 'object_id', 'uri']

admin.site.register(Visit, VisitAdmin)



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Adding model 'Visit'
        db.create_table('counter_visit', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('page_visited', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('visitor_ip', self.gf('django.db.models.fields.IPAddressField')(db_index=True, max_length=15, null=True, blank=True)),
            ('last_visit', self.gf('django.db.models.fields.DateTimeField')(null=True, blank=True)),
            ('visits', self.gf('django.db.models.fields.IntegerField')(default=0)),
            ('blocked', self.gf('django.db.models.fields.BooleanField')(default=False)),
        ))
        db.send_create_signal('counter', ['Visit'])


    def backwards(self, orm):
        
        # Deleting model 'Visit'
        db.delete_table('counter_visit')


    models = {
        'counter.visit': {
            'Meta': {'object_name': 'Visit'},
            'blocked': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'last_visit': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'page_visited': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'visitor_ip': ('django.db.models.fields.IPAddressField', [], {'db_index': 'True', 'max_length': '15', 'null': 'True', 'blank': 'True'}),
            'visits': ('django.db.models.fields.IntegerField', [], {'default': '0'})
        }
    }

    complete_apps = ['counter']



# -*- coding: utf-8 -*-
from visits import settings
from visits.models import Visit
from visits.utils import is_ignored

class CounterMiddleware:
    def process_request(self, request):
        Visit.objects.add_uri_visit(request, request.META["PATH_INFO"])

class BotVisitorMiddleware:
    def process_request(self, request):
        user_agent = request.META["HTTP_USER_AGENT"] or None
        if user_agent in settings.IGNORE_USER_AGENTS:
            request.META.setdefault("IS_BOT", True)
        request.META.setdefault("IS_BOT", False)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages
import visits

setup(
    name='django-visits',
    version = u":versiontools:visits:",
    description="Visit counter for Django",
    long_description=open('README.rst', 'r').read(),
    keywords='django, visit, counter, visitors',
    author='Jesús Espino García & Sultan Imanhodjaev',
    author_email='jespinog@gmail.com, sultan.imanhodjaev@gmail.com',
    url='https://bitbucket.org/jespino/django-visits',
    license='LGPL',
    include_package_data=True,
    packages=find_packages(),
    install_requires=[
        'distribute',
    ],
    setup_requires = [
        'versiontools >= 1.8',
    ],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "Intended Audience :: Information Technology",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Topic :: Internet :: Log Analysis",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content :: Page Counters",
        "Topic :: Internet :: WWW/HTTP :: WSGI :: Middleware",
        "Topic :: System :: Monitoring",
        "Topic :: Utilities",
    ]
)
